==> mls-0.1.tar.gz: mls-0.1/src/menu.cpp <==
#include <ncurses.h>

#include <algorithm>
#include <functional>

#include "menu.h"
#include "conf.h"
#include "drawutil.h"
#include "mlslocale.h"

#include "panel.h"

using namespace std;
using namespace MLS;

Menu::Menu()
{
	num = 0;
	win = NULL;
}

Menu::~Menu()
{
	if (win) delwin(win);
}

void Menu::Draw()
{	
	if (!win) return;
	
	setcol(conf.FuncN, win);	
	wmove(win, 0, 0);
	whline(win,' ', COLS);		
	for (int t=0; t<item.size(); t++)
		mvwprintw(win, 0, item[t].GetLocation() , "%-11s", item[t].GetName().c_str());	
	unsetcol(win);
	
	setcol(conf.FuncA, win);	
	for (int t=0; t<item.size(); t++)
		mvwaddch(win, 0, item[t].GetLocation(), item[t].GetName()[0]);	
	unsetcol(win);
	
	setcol(7, 0, win);	
	mvwprintw(win, 0, item[num].GetLocation()-1 , " %s ", item[num].GetName().c_str());
	unsetcol(win);
	
	setcol(11, 0, win);
	mvwaddch(win, 0, item[num].GetLocation(), item[num].GetName()[0]);	
	
	wrefresh(win);
	
	item[num].Draw();	
}

void Menu::Process(Panel &rp)
{
	int key;
	bool bRef=true;
	
	touchline(stdscr, 0, LINES);
	
	while(1)
	{
		// 필요할 때만 화면 원상복귀			
		if (bRef)
		{
			wtouchln(stdscr, 0, LINES, 1);
			wrefresh(stdscr);
			bRef=false;
		}		
		Draw();
		
		key = getch();
		
		if ('`' == key) break;
		
		if (13 == key)
		{
			wtouchln(stdscr, 0, LINES, 1);
			wrefresh(stdscr);
						
			rp.Command(item[num].GetCmd());
			break;			
		}
		
		if (KEY_LEFT == key)
		{
			num = (num+ item.size() -1) % item.size();
			bRef = true;
		}
		if (KEY_RIGHT == key) 
		{
			num = (num + 1) % item.size();		
			bRef = true;
		}
		
		if (KEY_UP == key)	item[num].CursorUp();
		if (KEY_DOWN == key) item[num].CursorDown();
		
	}
}

void Menu::Create()
{
	MenuCategory mls("Mls");
		mls.AddItem("About", "<about>");
		mls.AddItem("Help", "<help>");
		mls.AddItem("", "");
		mls.AddItem("Quit Mls", "<exit>");
	AddItem(mls);
	
	MenuCategory run("Run");
		run.AddItem("Dummy1", "");
	AddItem(run);
	
	MenuCategory files("File");
		files.AddItem("Refresh", "<refresh>");
		files.AddItem("Copy", "<copy>");
		files.AddItem("Move", "");
		files.AddItem("Rename", "<rename>");
		files.AddItem("Delete", "");
		files.AddItem("Chmod", "");
		files.AddItem("", "");
		files.AddItem("View", "<view>");
		files.AddItem("Edit", "<edit>");
		files.AddItem("FileFind", "");
		files.AddItem("", "");
		files.AddItem("Select all", "");
		files.AddItem("Invert select", "");
		files.AddItem("Select group", "");
	AddItem(files);
	
	MenuCategory dire("Directory");
		dire.AddItem("Mcd", "<mcd>");
		dire.AddItem("Qcd", "");
		dire.AddItem("", "");
		dire.AddItem("Make", "<mkdir>");
		dire.AddItem("Properties", "");
		
	AddItem(dire);
	
	MenuCategory arc("Archive");
		arc.AddItem("Into the Archive", "");
	AddItem(arc);
	
	MenuCategory op("Option");
		op.AddItem("Edit config file", "<editconfig>");
		op.AddItem("Edit colorset file", "<editcolorset>");		
	AddItem(op);
	
	if (win) delwin(win);	
	win = newwin(1, COLS, 0, 0);	
	
	CreateWindow();
}

void Menu::CreateWindow()
{
	for_each(item.begin(), item.end(), mem_fun_ref(&MenuCategory::Create));
}

void MenuCategory::Create()
{
	if (win) delwin(win);
	win = newwin(item.size() + 2, 24, 1, location-3);	
}

void MenuCategory::Draw()
{
	if (!win) return;
	
	// ----- 긋기		
	setcol(conf.FuncN, win);	
	wmove(win, 0, 0);
	waddch(win, ' ');
	waddch(win, ULCORNER);
	whline(win, HLINE, 20);
	mvwaddch(win, 0, 22, URCORNER);
	waddch(win, ' ');
			
	for(int c=0; c<item.size(); c++)
	{
		wmove(win, c+1, 0);
		waddch(win, ' ');
		waddch(win, VLINE);
		if (item[c].GetName().empty())
		{// 빈 문자열은 줄긋기
			unsetcol(win);
			setcol(0, conf.FuncN.back, win);
			whline(win, HLINE, 20);
			wmove(win, c+1, 22);			
			unsetcol(win);
			setcol(conf.FuncN, win);
		}
		else wprintw(win, " %-18.18s ", item[c].GetName().c_str());
		
		waddch(win, VLINE);	
		waddch(win, ' ');
	}
	
	// -------
	wmove(win, item.size()+1, 0);
	waddch(win, ' ');
	waddch(win, LLCORNER);
	whline(win, HLINE, 20);
	mvwaddch(win, item.size()+1, 22, LRCORNER);
	waddch(win, ' ');
		
	unsetcol(win);
	
	// 
	setcol(7, 0, win);
	wmove(win, index+1, 2);
	
	wprintw(win, " %-18.18s ", item[index].GetName().c_str());
		
	unsetcol(win);
	
	wrefresh(win);
}

void MLS::MenuCategory::CursorUp( )
{
	if (item.empty()) return;
	
	index = (index +item.size() - 1) % item.size();
	
	// 빈 스트링은 건너뛴다	
	// 메뉴전체가 빈 스트링일 경우 무한 루프 빠질위험 있음
	if (item[index].GetName().empty()) CursorUp();	
}

void MLS::MenuCategory::CursorDown( )
{
	if (item.empty()) return;
	
	index = (index + 1) % item.size();
	
	// 빈 스트링은 건너뛴다	
	// 메뉴전체가 빈 스트링일 경우 무한 루프 빠질위험 있음
	if (item[index].GetName().empty()) CursorDown();
}

==> mls-0.2.tar.gz: mls-0.2/src/menu.cpp <==
///	\file	menu.cpp
///	\brief	menu.cpp
#include <ncurses.h>

#include <algorithm>
#include <functional>

#include "mainframe.h"
#include "menu.h"
#include "conf.h"
#include "drawutil.h"
#include "mlslocale.h"

#include "panel.h"

using namespace std;
using namespace MLS;

///	\brief	생성자
Menu::Menu()
{
	num = 0;
	win = NULL;
}

/// \brief	소멸자
Menu::~Menu()
{
	// . 윈도우 삭제
	if (win) delwin(win);
}

///	\brief	메뉴를 화면에 그린다.
void Menu::Draw()
{	
	if (!win) return;
	
	// 주 메뉴를 그린다.
	// 예 "Mls  Run  File  Directory  Archive  Option" 메뉴
	setcol(conf.FuncN, win);	
	wmove(win, 0, 0);
	whline(win,' ', COLS);		
	for (int t=0; t<item.size(); t++)
		mvwprintw(win, 0, item[t].GetLocation() , "%-11s", item[t].GetName().c_str());	
	unsetcol(win);

	// 주 메뉴에서 첫글자를 노란글자로 쓴다.(구분)
	setcol(conf.FuncA, win);	
	for (int t=0; t<item.size(); t++)
		mvwaddch(win, 0, item[t].GetLocation(), item[t].GetName()[0]);	
	unsetcol(win);
	
	// 선택 된 메뉴에 검은색바탕에 글자를 씀
	setcol(7, 0, win);	
	mvwprintw(win, 0, item[num].GetLocation()-1 , " %s ", item[num].GetName().c_str());
	unsetcol(win);
	
	// 선택 된 메뉴에 첫번째 글자를 노란색으로 쓴다.
	setcol(11, 0, win);
	mvwaddch(win, 0, item[num].GetLocation(), item[num].GetName()[0]);	
	unsetcol(win);
	
	wrefresh(win);
	
	// 메뉴 뭉치를 그린다.
	item[num].Draw();	
}

///	\brief	메뉴에서 행해지는 행동들을 처리한다.
///	\param	rp		MLS Panel
void Menu::Process(Panel &rp)
{
	int key;
	bool bRef=true;
	
	//touchline(stdscr, 0, LINES);
	
	while(1)
	{
		// 필요할 때만 화면 원상복귀			
		if (bRef)
		{
		//	wtouchln(stdscr, 0, LINES, 1);
		//	wrefresh(stdscr);
			g_MainFrame->Draw();
			bRef=false;
		}
			
		
		// .. Menu 화면을 그린다.
		Draw();
		
		// .. 키입력을 받는다.
		key = getch();
		
		// .. '`'입력을 받으면 메뉴를 없애고 메뉴를 없앤다.
		if ('`' == key) break;
		
		// .. 엔터를 입력하면 해당 메뉴의 명령을 실행한다.
		if (13 == key)
		{
			wtouchln(stdscr, 0, LINES, 1);
			wrefresh(stdscr);
			
			g_MainFrame->tCommand.Execute(item[num].GetCmd());
			break;			
		}
		
		// .. 왼쪽 화살표를 누르면 왼쪽에 있는 메뉴로 이동한다.
		if (KEY_LEFT == key)
		{
			num = (num+ item.size() -1) % item.size();
			bRef = true;
		}

		// .. 오른쪽 화살표를 누르면 오른쪽에 있는 메뉴로 이동한다.
		if (KEY_RIGHT == key) 
		{
			num = (num + 1) % item.size();		
			bRef = true;
		}
		
		// .. 위쪽 화살표를 누르면 메뉴item위치 위로 변경
		if (KEY_UP == key)	item[num].CursorUp();
		// .. 아래쪽 화살표를 누르면 메뉴item위치 아래로 변경
		if (KEY_DOWN == key) item[num].CursorDown();
		
	}
}

///	\brief	메뉴를 생성한다.
void Menu::Create()
{
	MenuCategory mls("Mls");
		mls.AddItem("About", "<about>");
		mls.AddItem("Help", "<help>");
		mls.AddItem("", "");
		mls.AddItem("Quit Mls", "<exit>");
	AddItem(mls);
	
	MenuCategory run("Run");
		run.AddItem("Execute", "<enter>");
	AddItem(run);
	
	MenuCategory files("File");
		files.AddItem("Refresh", "<refresh>");
		files.AddItem("Copy", "<copy>");
		files.AddItem("Move", "<move>");
		files.AddItem("Rename", "<rename>");
		files.AddItem("Delete", "<delete>");
		files.AddItem("Chmod", "");
		files.AddItem("", "");
		files.AddItem("View", "<view>");
		files.AddItem("Edit", "<edit>");
		files.AddItem("FileFind", "");
		files.AddItem("", "");
		files.AddItem("Select all", "");
		files.AddItem("Invert select", "");
		files.AddItem("Select group", "");
	AddItem(files);
	
	MenuCategory dire("Directory");
		dire.AddItem("Mcd", "<mcd>");
		dire.AddItem("Qcd", "");
		dire.AddItem("", "");
		dire.AddItem("Make", "<mkdir>");
		dire.AddItem("Properties", "");
		
	AddItem(dire);
	
	MenuCategory arc("Archive");
		arc.AddItem("Into the Archive", "");
	AddItem(arc);
	
	MenuCategory op("Option");
		op.AddItem("Edit config file", "<editconfig>");
		op.AddItem("Edit colorset file", "<editcolorset>");		
	AddItem(op);
	
	if (win) delwin(win);	
	win = newwin(1, COLS, 0, 0);	
	
	CreateWindow();
}

///	\brief	메뉴윈도우를 만들다.
void Menu::CreateWindow()
{
	for_each(item.begin(), item.end(), mem_fun_ref(&MenuCategory::Create));
}

///	\brief	메뉴 몽치를 생성한다.
void MenuCategory::Create()
{
	if (win) delwin(win);
	win = newwin(item.size() + 2, 24, 1, location-3);// (wy, wx, y, x)
}

///	\brief	메뉴 뭉치를 화면에 그린다.
void MenuCategory::Draw()
{
	if (!win) return;
	
	// ----- 긋기		
	setcol(conf.FuncN, win);	
	wmove(win, 0, 0);
	waddch(win, ' ');
	waddch(win, ULCORNER);
	whline(win, HLINE, 20);
	mvwaddch(win, 0, 22, URCORNER);
	waddch(win, ' ');
			
	for(int c=0; c<item.size(); c++)
	{
		wmove(win, c+1, 0);
		waddch(win, ' ');
		waddch(win, VLINE);
		if (item[c].GetName().empty())
		{// 빈 문자열은 줄긋기
			unsetcol(win);
			setcol(0, conf.FuncN.back, win);
			whline(win, HLINE, 20);
			wmove(win, c+1, 22);			
			unsetcol(win);
			setcol(conf.FuncN, win);
		}
		else wprintw(win, " %-18.18s ", item[c].GetName().c_str());
		
		waddch(win, VLINE);	
		waddch(win, ' ');
	}
	
	// ----- 긋기		
	wmove(win, item.size()+1, 0);
	waddch(win, ' ');
	waddch(win, LLCORNER);
	whline(win, HLINE, 20);
	mvwaddch(win, item.size()+1, 22, LRCORNER);
	waddch(win, ' ');
		
	unsetcol(win);
	
	// . 선택 된 item를 반전 시켜 보여줌
	setcol(7, 0, win);
	wmove(win, index+1, 2);
	wprintw(win, " %-18.18s ", item[index].GetName().c_str());
		
	unsetcol(win);
	
	wrefresh(win);
}

///	\brief	현재 선택 되어 있는 메뉴 항목위치을 위로 하나 이동시킴
void MLS::MenuCategory::CursorUp( )
{
	if (item.empty()) return;
	
	index = (index +item.size() - 1) % item.size();
	
	// 빈 스트링은 건너뛴다	
	// 메뉴전체가 빈 스트링일 경우 무한 루프 빠질위험 있음
	if (item[index].GetName().empty()) CursorUp();	
}

///	\brief	현재 선택 되어 있는 메뉴 항목위치을 아래로 하나 이동시킴
void MLS::MenuCategory::CursorDown( )
{
	if (item.empty()) return;
	
	index = (index + 1) % item.size();
	
	// 빈 스트링은 건너뛴다	
	// 메뉴전체가 빈 스트링일 경우 무한 루프 빠질위험 있음
	if (item[index].GetName().empty()) CursorDown();
}
